package com.nisum.order_producer.service;

import com.nisum.order_producer.dto.OrderDto;
import com.nisum.order_producer.entity.Order;
import com.nisum.order_producer.mapper.OrderMapper;
import com.nisum.order_producer.repository.OrderProducerRepository;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.reactive.function.client.WebClient;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.time.LocalDateTime;

@Service
public class OrderProducerServiceImpl implements OrderProducerService {
    @Value("${consumer.base-url}")
    private String baseUrl;

    @Autowired
    private OrderProducerRepository orderProducerRepository;

    @Autowired
    private WebClient webClient;

    @Autowired
    private OrderMapper orderMapper;
    private static final Logger logger = LoggerFactory.getLogger(OrderProducerServiceImpl.class);



    @Override
    public Mono<Order> saveOrder(Order order) {

        logger.info("Received request to save order: {}", order);

        //Order order = orderMapper.toEntity(orderDto);
        order.setCreatedDate(LocalDateTime.now());
        order.setModifiedDate(LocalDateTime.now());

       // logger.info("Mapped Order DTO to Entity: {}", order);

        return orderProducerRepository
                .save(order)
                .doOnSuccess(savedOrder -> logger.info("Order saved successfully: {}", savedOrder))
                .flatMap(savedOrder -> sendOrderToConsumer(savedOrder)
                        .thenReturn(savedOrder))
                .doOnError(error -> logger.error("Error saving order: {}", error.getMessage()));
    }

    @Override
    public Flux<Order> getAllOrder() {
        logger.info("Fetching all orders...");
        return orderProducerRepository
                .findAll()
                .doOnComplete(() -> logger.info("Fetched all orders successfully"))
                .doOnError(error -> logger.error("Error fetching orders: {}", error.getMessage()));
    }

    @Override
    public Mono<Order> getByOrderId(String id) {
        logger.info("Fetching order with ID: {}", id);
        return orderProducerRepository
                .findById(id)
                .doOnSuccess(orderDto -> logger.info("Order found: {}", orderDto))
                .switchIfEmpty(Mono.error(new RuntimeException("Order not found with id " + id)))
                .doOnError(error -> logger.error("Error fetching order by ID {}: {}", id, error.getMessage()));
    }

    private Mono<Void> sendOrderToConsumer(Order order){
        logger.info("Sending order to consumer: {}", order);
        return webClient
                .post()
                .uri(baseUrl + "/saveOrder")
                .bodyValue(order)
                .retrieve()
                .bodyToMono(Void.class)
                .doOnSuccess(response -> logger.info("Successfully sent order to consumer"))
                .onErrorResume(ex -> {
                    logger.error("Error sending data to consumer: {}", ex.getMessage());
                    return Mono.empty();
                });
    }
}
